@@ -1,21 +1,11 @@
 /*
- * Copyright (C) 2011-2014  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2011-2016  Internet Systems Consortium, Inc. ("ISC")
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-/* $Id$ */
-
 /* ! \file */
 
 #include <config.h>
@@ -25,14 +15,19 @@
 #include <stdio.h>
 #include <string.h>
 
+#include <isc/hash.h>
+
 #include <isc/crc64.h>
 #include <isc/hmacmd5.h>
 #include <isc/hmacsha.h>
 #include <isc/md5.h>
 #include <isc/sha1.h>
 #include <isc/util.h>
+#include <isc/print.h>
 #include <isc/string.h>
 
+#include <pk11/site.h>
+
 /*
  * Test data from RFC6234
  */
@@ -40,9 +35,8 @@
 unsigned char digest[ISC_SHA512_DIGESTLENGTH];
 unsigned char buffer[1024];
 const char *s;
-char str[ISC_SHA512_DIGESTLENGTH];
+char str[2 * ISC_SHA512_DIGESTLENGTH + 3];
 unsigned char key[20];
-int i = 0;
 
 isc_result_t
 tohexstr(unsigned char *d, unsigned int len, char *out);
@@ -52,7 +46,7 @@ tohexstr(unsigned char *d, unsigned int len, char *out);
  * Postcondition: A String representation of the given hexadecimal number is
  *   placed into the array *out
  *
- * 'out' MUST point to an array of at least len / 2 + 1
+ * 'out' MUST point to an array of at least len * 2 + 1
  *
  * Return values: ISC_R_SUCCESS if the operation is sucessful
  */
@@ -95,6 +89,7 @@ ATF_TC_HEAD(isc_sha1, tc) {
 }
 ATF_TC_BODY(isc_sha1, tc) {
 	isc_sha1_t sha1;
+	int i;
 
 	UNUSED(tc);
 
@@ -217,13 +212,13 @@ ATF_TC_BODY(isc_sha1, tc) {
 	}
 }
 
-
 ATF_TC(isc_sha224);
 ATF_TC_HEAD(isc_sha224, tc) {
 	atf_tc_set_md_var(tc, "descr", "sha224 examples from RFC4634");
 }
 ATF_TC_BODY(isc_sha224, tc) {
 	isc_sha224_t sha224;
+	int i;
 
 	UNUSED(tc);
 
@@ -348,7 +343,6 @@ ATF_TC_BODY(isc_sha224, tc) {
 
 		testcase++;
 	}
-
 }
 
 ATF_TC(isc_sha256);
@@ -357,6 +351,7 @@ ATF_TC_HEAD(isc_sha256, tc) {
 }
 ATF_TC_BODY(isc_sha256, tc) {
 	isc_sha256_t sha256;
+	int i;
 
 	UNUSED(tc);
 
@@ -480,7 +475,6 @@ ATF_TC_BODY(isc_sha256, tc) {
 
 		testcase++;
 	}
-
 }
 
 ATF_TC(isc_sha384);
@@ -489,6 +483,7 @@ ATF_TC_HEAD(isc_sha384, tc) {
 }
 ATF_TC_BODY(isc_sha384, tc) {
 	isc_sha384_t sha384;
+	int i;
 
 	UNUSED(tc);
 
@@ -626,7 +621,6 @@ ATF_TC_BODY(isc_sha384, tc) {
 
 		testcase++;
 	}
-
 }
 
 ATF_TC(isc_sha512);
@@ -635,6 +629,7 @@ ATF_TC_HEAD(isc_sha512, tc) {
 }
 ATF_TC_BODY(isc_sha512, tc) {
 	isc_sha512_t sha512;
+	int i;
 
 	UNUSED(tc);
 
@@ -773,15 +768,16 @@ ATF_TC_BODY(isc_sha512, tc) {
 
 		testcase++;
 	}
-
 }
 
+#ifndef PK11_MD5_DISABLE
 ATF_TC(isc_md5);
 ATF_TC_HEAD(isc_md5, tc) {
 	atf_tc_set_md_var(tc, "descr", "md5 example from RFC1321");
 }
 ATF_TC_BODY(isc_md5, tc) {
 	isc_md5_t md5;
+	int i;
 
 	UNUSED(tc);
 
@@ -847,6 +843,7 @@ ATF_TC_BODY(isc_md5, tc) {
 		testcase++;
 	}
 }
+#endif
 
 /* HMAC-SHA1 test */
 ATF_TC(isc_hmacsha1);
@@ -1643,6 +1640,7 @@ ATF_TC_BODY(isc_hmacsha512, tc) {
 }
 
 
+#ifndef PK11_MD5_DISABLE
 /* HMAC-MD5 Test */
 ATF_TC(isc_hmacmd5);
 ATF_TC_HEAD(isc_hmacmd5, tc) {
@@ -1784,6 +1782,7 @@ ATF_TC_BODY(isc_hmacmd5, tc) {
 		test_key++;
 	}
 }
+#endif
 
 /* CRC64 Test */
 ATF_TC(isc_crc64);
@@ -1792,6 +1791,7 @@ ATF_TC_HEAD(isc_crc64, tc) {
 }
 ATF_TC_BODY(isc_crc64, tc) {
 	isc_uint64_t crc;
+	int i;
 
 	UNUSED(tc);
 
@@ -1846,23 +1846,151 @@ ATF_TC_BODY(isc_crc64, tc) {
 	}
 }
 
+ATF_TC(isc_hash_function);
+ATF_TC_HEAD(isc_hash_function, tc) {
+	atf_tc_set_md_var(tc, "descr", "Hash function test");
+}
+ATF_TC_BODY(isc_hash_function, tc) {
+	unsigned int h1;
+	unsigned int h2;
+
+	UNUSED(tc);
+
+	/* Incremental hashing */
+
+	h1 = isc_hash_function(NULL, 0, ISC_TRUE, NULL);
+	h1 = isc_hash_function("This ", 5, ISC_TRUE, &h1);
+	h1 = isc_hash_function("is ", 3, ISC_TRUE, &h1);
+	h1 = isc_hash_function("a long test", 12, ISC_TRUE, &h1);
+
+	h2 = isc_hash_function("This is a long test", 20,
+			       ISC_TRUE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	/* Immutability of hash function */
+	h1 = isc_hash_function(NULL, 0, ISC_TRUE, NULL);
+	h2 = isc_hash_function(NULL, 0, ISC_TRUE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	/* Hash function characteristics */
+	h1 = isc_hash_function("Hello world", 12, ISC_TRUE, NULL);
+	h2 = isc_hash_function("Hello world", 12, ISC_TRUE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	/* Case */
+	h1 = isc_hash_function("Hello world", 12, ISC_FALSE, NULL);
+	h2 = isc_hash_function("heLLo WorLd", 12, ISC_FALSE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	/* Unequal */
+	h1 = isc_hash_function("Hello world", 12, ISC_TRUE, NULL);
+	h2 = isc_hash_function("heLLo WorLd", 12, ISC_TRUE, NULL);
+
+	ATF_CHECK(h1 != h2);
+}
+
+ATF_TC(isc_hash_function_reverse);
+ATF_TC_HEAD(isc_hash_function_reverse, tc) {
+	atf_tc_set_md_var(tc, "descr", "Reverse hash function test");
+}
+ATF_TC_BODY(isc_hash_function_reverse, tc) {
+	unsigned int h1;
+	unsigned int h2;
+
+	UNUSED(tc);
+
+	/* Incremental hashing */
+
+	h1 = isc_hash_function_reverse(NULL, 0, ISC_TRUE, NULL);
+	h1 = isc_hash_function_reverse("\000", 1, ISC_TRUE, &h1);
+	h1 = isc_hash_function_reverse("\003org", 4, ISC_TRUE, &h1);
+	h1 = isc_hash_function_reverse("\007example", 8, ISC_TRUE, &h1);
+
+	h2 = isc_hash_function_reverse("\007example\003org\000", 13,
+				       ISC_TRUE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	/* Immutability of hash function */
+	h1 = isc_hash_function_reverse(NULL, 0, ISC_TRUE, NULL);
+	h2 = isc_hash_function_reverse(NULL, 0, ISC_TRUE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	/* Hash function characteristics */
+	h1 = isc_hash_function_reverse("Hello world", 12, ISC_TRUE, NULL);
+	h2 = isc_hash_function_reverse("Hello world", 12, ISC_TRUE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	/* Case */
+	h1 = isc_hash_function_reverse("Hello world", 12, ISC_FALSE, NULL);
+	h2 = isc_hash_function_reverse("heLLo WorLd", 12, ISC_FALSE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	/* Unequal */
+	h1 = isc_hash_function_reverse("Hello world", 12, ISC_TRUE, NULL);
+	h2 = isc_hash_function_reverse("heLLo WorLd", 12, ISC_TRUE, NULL);
+
+	ATF_CHECK(h1 != h2);
+}
+
+ATF_TC(isc_hash_initializer);
+ATF_TC_HEAD(isc_hash_initializer, tc) {
+	atf_tc_set_md_var(tc, "descr", "Hash function initializer test");
+}
+ATF_TC_BODY(isc_hash_initializer, tc) {
+	unsigned int h1;
+	unsigned int h2;
+
+	UNUSED(tc);
+
+	h1 = isc_hash_function("Hello world", 12, ISC_TRUE, NULL);
+	h2 = isc_hash_function("Hello world", 12, ISC_TRUE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	isc_hash_set_initializer(isc_hash_get_initializer());
+
+	/* Hash value must not change */
+	h2 = isc_hash_function("Hello world", 12, ISC_TRUE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+}
+
 /*
  * Main
  */
 ATF_TP_ADD_TCS(tp) {
+	/*
+	 * Tests of hash functions, including isc_hash and the
+	 * various cryptographic hashes.
+	 */
+	ATF_TP_ADD_TC(tp, isc_hash_function);
+	ATF_TP_ADD_TC(tp, isc_hash_function_reverse);
+	ATF_TP_ADD_TC(tp, isc_hash_initializer);
+#ifndef PK11_MD5_DISABLE
 	ATF_TP_ADD_TC(tp, isc_hmacmd5);
+#endif
 	ATF_TP_ADD_TC(tp, isc_hmacsha1);
 	ATF_TP_ADD_TC(tp, isc_hmacsha224);
 	ATF_TP_ADD_TC(tp, isc_hmacsha256);
 	ATF_TP_ADD_TC(tp, isc_hmacsha384);
 	ATF_TP_ADD_TC(tp, isc_hmacsha512);
+#ifndef PK11_MD5_DISABLE
 	ATF_TP_ADD_TC(tp, isc_md5);
+#endif
 	ATF_TP_ADD_TC(tp, isc_sha1);
 	ATF_TP_ADD_TC(tp, isc_sha224);
 	ATF_TP_ADD_TC(tp, isc_sha256);
 	ATF_TP_ADD_TC(tp, isc_sha384);
 	ATF_TP_ADD_TC(tp, isc_sha512);
 	ATF_TP_ADD_TC(tp, isc_crc64);
+
 	return (atf_no_error());
 }
-
